@@ -15,7 +15,7 @@ import akernel.IPython
 from akernel.IPython import core
 from .connect import connect_channel
 from .message import receive_message, send_message, create_message, check_message
-from .code import make_async
+from .execution import pre_execute
 from .traceback import get_traceback
 from ._version import __version__
 
@@ -41,7 +41,7 @@ class Kernel:
     restart: bool
     key: str
     comm_manager: CommManager
-    kernel_name: str
+    kernel_mode: str
     running_cells: Dict[int, asyncio.Task]
     task_i: int
     execution_count: int
@@ -51,14 +51,14 @@ class Kernel:
 
     def __init__(
         self,
-        kernel_name: str,
+        kernel_mode: str,
         connection_file: str,
     ):
         global KERNEL
         KERNEL = self
         self.comm_manager = CommManager()
         self.loop = asyncio.get_event_loop()
-        self.kernel_name = kernel_name
+        self.kernel_mode = kernel_mode
         self.running_cells = {}
         self.task_i = 0
         self.execution_count = 1
@@ -70,6 +70,9 @@ class Kernel:
             "_": None,
         }
         self.locals = {}
+        if kernel_mode == "react":
+            code = "import ipyx; globals()['ipyx'] = ipyx"
+            exec(code, self.globals, self.locals)
         with open(connection_file) as f:
             self.connection_cfg = json.load(f)
         self.key = cast(str, self.connection_cfg["key"])
@@ -173,27 +176,26 @@ class Kernel:
                     content={"code": code, "execution_count": self.execution_count},
                 )
                 send_message(msg, self.iopub_channel, self.key)
-                return_value, async_code = make_async(code)
-                try:
-                    exec(async_code, self.globals, self.locals)
-                except Exception as e:
+                react = self.kernel_mode == "react"
+                traceback, exception = pre_execute(
+                    code, self.globals, self.locals, self.execution_count, react=react
+                )
+                if traceback:
                     self.finish_execution(
                         idents,
                         parent_header,
                         self.execution_count,
-                        exception=e,
-                        code=code,
-                        return_value=return_value,
+                        traceback=traceback,
+                        exception=exception,
                     )
                 else:
                     task = asyncio.create_task(
-                        self.execute_code(
+                        self.execute_and_finish(
                             idents,
                             parent_header,
                             self.task_i,
                             self.execution_count,
                             code,
-                            return_value,
                         )
                     )
                     self.running_cells[self.task_i] = task
@@ -255,27 +257,29 @@ class Kernel:
                         "_": None,
                     }
                     self.locals = {}
+                    if self.kernel_mode == "react":
+                        code = "import ipyx; globals()['ipyx'] = ipyx"
+                        exec(code, self.globals, self.locals)
                     self.execution_count = 1
                 self.stop.set()
 
-    async def execute_code(
+    async def execute_and_finish(
         self,
         idents: List[bytes],
         parent_header: Dict[str, Any],
         task_i: int,
         execution_count: int,
         code: str,
-        return_value: bool,
     ) -> None:
         PARENT_HEADER_VAR.set(parent_header)
-        traceback, exception = None, None
+        traceback, exception = [], None
         try:
             result = await self.locals["__async_cell__"]()
         except KeyboardInterrupt:
             self.interrupt()
         except Exception as e:
             exception = e
-            traceback = get_traceback(code, return_value)
+            traceback = get_traceback(code, e, execution_count)
         else:
             if result is not None:
                 self.globals["_"] = result
@@ -309,52 +313,26 @@ class Kernel:
         execution_count: Optional[int],
         exception: Optional[Exception] = None,
         no_exec: bool = False,
-        traceback: Optional[List[str]] = None,
-        code: Optional[str] = None,
-        return_value: bool = False,
+        traceback: List[str] = [],
     ) -> None:
         if no_exec:
             status = "aborted"
         else:
-            if traceback is None and exception is None:
-                status = "ok"
-            else:
+            if traceback:
                 status = "error"
                 assert exception is not None
-                assert exception.args is not None
-                if type(exception) is SyntaxError:
-                    assert exception.lineno is not None
-                    assert exception.text is not None
-                    assert exception.offset is not None
-                    if exception.filename == "<string>":
-                        assert code is not None
-                        code_lines = code.splitlines()
-                        lineno = exception.lineno - 2
-                        if return_value and lineno == len(code_lines) + 1:
-                            lineno -= 1
-                        text = code_lines[lineno - 1].rstrip()
-                        offset = exception.offset - 5
-                    else:
-                        lineno = exception.lineno
-                        text = exception.text.rstrip()
-                        offset = exception.offset - 1
-                    traceback = [
-                        f"File {exception.filename}, line {lineno}:",
-                        text,
-                        offset * " " + "^",
-                    ]
-                assert traceback is not None
                 msg = create_message(
                     "error",
                     parent_header=parent_header,
                     content={
                         "ename": type(exception).__name__,
                         "evalue": exception.args[0],
-                        "traceback": traceback
-                        + [f"{type(exception).__name__}: {exception.args[0]}"],
+                        "traceback": traceback,
                     },
                 )
                 send_message(msg, self.iopub_channel, self.key)
+            else:
+                status = "ok"
         msg = self.create_message(
             "execute_reply",
             parent_header=parent_header,
